/* G Calendar
 * Copyright (C) 1998 Cesar Miquel, Shawn T. Amundson, Mattias Grnlund
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtk/cc/gtk.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#define DEF_PAD 10
#define DEF_PAD_SMALL 5

#define TM_YEAR_BASE 1900


typedef struct _CalendarData {
  GtkCheckButton *flag_checkboxes[5];
  gboolean  settings[5];
  gchar     *font;
  GtkFontSelectionDialog *font_dialog;
  GtkCalendar *window;
  GtkLabel *selected;
  GtkLabel *selected_double_click;
  GtkLabel *month;
} CalendarData;

void create_calendar();
gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data);
void destroy (GtkWidget *widget, gpointer data);
void day_selected_double_click (GtkWidget *widget, gpointer data);
int main(int argc, char *argv[]);
void calendar_month_changed (GtkWidget *widget, CalendarData *data);
void calendar_day_selected (GtkWidget *widget, CalendarData *data);
void calendar_day_selected_double_click (GtkWidget *widget, CalendarData *data);


void calendar_set_flags(CalendarData *calendar);
void calendar_toggle_flag(GtkCheckButton * toggle, CalendarData *calendar);
void calendar_font_selection_ok(GtkWidget * button, CalendarData *calendar);
void calendar_font_selection_destroy(GtkWidget * button, GtkWidget widget);
void calendar_select_font(GtkWidget * button, CalendarData *calendar);
void calendar_create_window_destroy(GtkWidget * ignore, CalendarData *calendar);
void calendar_create_window(GtkWidget * ignored, CalendarData * calendar);
void calendar_window_destroy(GtkWidget * ignore, CalendarData *calendar);
void create_calendar();

/*
 * GtkCalendar
 */

void
calendar_month_changed (GtkWidget *widget, CalendarData *data)
{
  char buffer[256];
  struct tm tm;
  time_t time;
  memset (&tm, 0, sizeof (tm));
  data->window->get_date_ ((guint*)&tm.tm_year, (guint*)&tm.tm_mon, (guint*)&tm.tm_mday);
  tm.tm_year -= TM_YEAR_BASE;
  time = mktime(&tm);
  strftime (buffer, 255, "%x", gmtime(&time));
  data->month->set_text_ (buffer);
}

void
calendar_day_selected (GtkWidget *widget, CalendarData *data)
{
  char buffer[256];
  struct tm tm;
  time_t time;
  memset (&tm, 0, sizeof (tm));
  data->window->get_date_ ((guint*)&tm.tm_year, (guint*)&tm.tm_mon, (guint*)&tm.tm_mday);
  tm.tm_year -= TM_YEAR_BASE;
  time = mktime(&tm);
  strftime (buffer, 255, "%x", gmtime(&time));
  data->selected->set_text_ (buffer);
}

void
calendar_day_selected_double_click (GtkWidget *widget, CalendarData *data)
{
  char buffer[256];
  struct tm tm;
  time_t time;
  memset (&tm, 0, sizeof (tm));
  data->window->get_date_ ((guint*)&tm.tm_year, (guint*)&tm.tm_mon, (guint*)&tm.tm_mday);
  tm.tm_year -= TM_YEAR_BASE;
  time = mktime(&tm);
  strftime (buffer, 255, "%x", gmtime(&time));
  data->selected_double_click->set_text_ (buffer);
}

enum {
  calendar_show_header,
  calendar_show_days,
  calendar_month_change, 
  calendar_show_week,
  calendar_monday_first
};

void
calendar_set_flags(CalendarData *calendar)
{
  gint i;
  GtkCalendarDisplayOptions options=0;
  for (i=0;i<5;i++) 
    if (calendar->settings[i])
      {
	options=options + (1<<i);
      }
  if (calendar->window)
    calendar->window->display_options_ (options);
}

void 
calendar_toggle_flag(GtkCheckButton * toggle, CalendarData *calendar)
{
  gint i;
  gint j;
  j=0;
  for (i=0; i<5; i++)
    if (calendar->flag_checkboxes[i] == toggle)
      j = i;

  calendar->settings[j]=!calendar->settings[j];
  calendar_set_flags(calendar);
  
}

void
calendar_font_selection_ok(GtkWidget * button, CalendarData *calendar)
{
  GtkStyle *style;
  GdkFont  *font;

  calendar->font =  calendar->font_dialog->get_font_name_ ();
  if (calendar->window)
    {
      font = calendar->font_dialog->get_font_ ();
      if (font) 
	{
	  style = calendar->window->get_style_ ()->copy_ ();
	  gdk_font_unref (style->font);
	  style->font = font;
	  gdk_font_ref (style->font);
	  calendar->window->set_style_ (style);
	}
    }
}

void
calendar_select_font(GtkWidget * button, CalendarData *calendar)
{
  GtkFontSelectionDialog *window;

  if (!calendar->font_dialog) {
    window = GtkFontSelectionDialog::_new ("Font Selection Dialog");
    g_return_if_fail(GTK_IS_FONT_SELECTION_DIALOG(window));
    calendar->font_dialog = window;
    
    window->set_position_ (GTK_WIN_POS_MOUSE);
    
    window->connect_ ("destroy", GTK_SIG gtk_widget_destroyed, &calendar->font_dialog);
    
    window->ok_button->connect_ ("clicked", 
        			GTK_SIG calendar_font_selection_ok, calendar);
    window->cancel_button->connect_object_ ("clicked",
        			GTK_SIG gtk_widget_destroy, calendar->font_dialog);
  }

  window = calendar->font_dialog;
  if (!GTK_WIDGET_VISIBLE (window))
    window->show_ ();
  else
    window->destroy_ ();

}

void 
create_calendar()
{
  GtkWindow *window;
  GtkVBox *vbox, *vbox2, *vbox3;
  GtkHBox *hbox;
  GtkHButtonBox *hbbox;
  GtkCalendar *calendar;
  GtkCheckButton *toggle;
  GtkButton *button;
  GtkFrame *frame;
  GtkSeparator *separator;
  GtkLabel *label;
  GtkButtonBox *bbox;
  static CalendarData calendar_data;
  gint i;
  
  struct {
    char *label;
  } flags[] =
    {
      { "Show Heading" },
      { "Show Day Names" },
      { "No Month Change" },
      { "Show Week Numbers" },
      { "Week Start Monday" }
    };

  
  calendar_data.window = NULL;
  calendar_data.font = NULL;
  calendar_data.font_dialog = NULL;

  for (i=0; i<5; i++) {
    calendar_data.settings[i]=0;
  }

  window = GtkWindow::_new (GTK_WINDOW_TOPLEVEL);
  window->set_border_width_ (5);
  window->connect_ ("destroy", GTK_SIG gtk_main_quit, NULL);
  window->connect_ ("delete-event", GTK_SIG gtk_false, NULL);

  window->set_policy_ (FALSE, FALSE, TRUE);

  vbox = GtkVBox::_new (FALSE, DEF_PAD);
  window->add_ (vbox);

  /*
   * The top part of the window, Calendar, flags and fontsel.
   */

  hbox = GtkHBox::_new (FALSE, DEF_PAD);
  vbox->pack_start_ (hbox, TRUE, TRUE, DEF_PAD);
  hbbox = GtkHButtonBox::_new ();
  hbox->pack_start_ (hbbox, FALSE, FALSE, DEF_PAD);
  hbbox->set_layout_ (GTK_BUTTONBOX_SPREAD);
  hbbox->set_spacing_ (5);

  /* Calendar widget */
  frame = GtkFrame::_new ("Calendar");
  hbbox->pack_start_ (frame, TRUE, TRUE, DEF_PAD);
  calendar = GtkCalendar::_new ();
  calendar_data.window = calendar;
  calendar_set_flags (&calendar_data);
  calendar->mark_day_ (19);	
  frame->add_ (calendar);
  calendar->connect_ ("month_changed", GTK_SIG calendar_month_changed, &calendar_data);
  calendar->connect_ ("day_selected", GTK_SIG calendar_day_selected, &calendar_data);
  calendar->connect_ ("day_selected_double_click", 
      			GTK_SIG calendar_day_selected_double_click, &calendar_data);


  separator = GtkVSeparator::_new ();
  hbox->pack_start_ (separator, FALSE, TRUE, 0);

  vbox2 = GtkVBox::_new (FALSE, DEF_PAD);
  hbox->pack_start_ (vbox2, FALSE, FALSE, DEF_PAD);
  
  /* Build the Right frame with the flags in */ 

  frame = GtkFrame::_new ("Flags");
  vbox2->pack_start_ (frame, TRUE, TRUE, DEF_PAD);
  vbox3 = GtkVBox::_new (TRUE, DEF_PAD_SMALL);
  frame->add_ (vbox3);

  for (i = 0; i < 5; i++)
    {
      toggle = GtkCheckButton::_new_with_label (flags[i].label);
      toggle->connect_ ("toggled", GTK_SIG calendar_toggle_flag, &calendar_data);
      vbox3->pack_start_ (toggle, TRUE, TRUE, 0);
      calendar_data.flag_checkboxes[i]=toggle;
    }
  /* Build the right font-button */ 
  button = GtkButton::_new_with_label ("Font...");
  button->connect_ ("clicked", GTK_SIG calendar_select_font, &calendar_data);
  vbox2->pack_start_ (button, FALSE, FALSE, 0);

  /*
   *  Build the Signal-event part.
   */

  frame = GtkFrame::_new ("Signal events");
  vbox->pack_start_ (frame, TRUE, TRUE, DEF_PAD);
  vbox2 = GtkVBox::_new (TRUE, DEF_PAD_SMALL);
  frame->add_ (vbox2);
  
  hbox = GtkHBox::_new (FALSE, 5);
  vbox2->pack_start_ (hbox, FALSE, TRUE, 0);
  hbox->pack_start_ ( (GtkLabel::_new ("Day selected")), FALSE, TRUE, 0);
  hbox->pack_start_ ( (calendar_data.selected= GtkLabel::_new ("")), FALSE, TRUE, 0);

  hbox = GtkHBox::_new (FALSE, 5);
  vbox2->pack_start_ (hbox, FALSE, TRUE, 0);
  hbox->pack_start_ ( (GtkLabel::_new ("Day selected double click")), FALSE, TRUE, 0);
  hbox->pack_start_ ( (calendar_data.selected_double_click= GtkLabel::_new ("")), FALSE, TRUE, 0);

  hbox = GtkHBox::_new (FALSE, 5);
  vbox2->pack_start_ (hbox, FALSE, TRUE, 0);
  hbox->pack_start_ ( (GtkLabel::_new ("Month change:")), FALSE, TRUE, 0);
  hbox->pack_start_ ( (calendar_data.month= GtkLabel::_new ("")), FALSE, TRUE, 0);
  
  bbox = GtkHButtonBox::_new ();
  vbox->pack_start_ (bbox, FALSE, FALSE, 0);
  bbox->set_layout_ (GTK_BUTTONBOX_END);

  button = GtkButton::_new_with_label ("Close");
  button->connect_ ("clicked", GTK_SIG gtk_main_quit, NULL);
  bbox->add_ (button);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  button->grab_default_ ();

  window->show_all_ ();
}


int
main(int argc, char *argv[])
{
  gtk_set_locale ();
  gtk_init (&argc, &argv);
  create_calendar();
  gtk_main ();
  return 0;
}
