/* example-start clist clist.c */

#include <gtk/cc/gtk.h>

/* User clicked the "Add List" button. */
void button_add_clicked( GtkCList* clist )
{
    int indx;
 
    /* Something silly to add to the list. 4 rows of 2 columns each */
    gchar *drink[4][2] = { { "Milk",    "3 Oz" },
                           { "Water",   "6 l" },
                           { "Carrots", "2" },
                           { "Snakes",  "55" } };

    /* Here we do the actual adding of the text. It's done once for
     * each row.
     */
    for ( indx=0 ; indx < 4 ; indx++ )
	clist->append_ (drink[indx]);

    return;
}

/* User clicked the "Clear List" button. */
void button_clear_clicked( GtkCList* clist )
{
    /* Clear the list using gtk_clist_clear. This is much faster than
     * calling gtk_clist_remove once for each row.
     */
    clist->clear_ ();

    return;
}

/* The user clicked the "Hide/Show titles" button. */
void button_hide_show_clicked( GtkCList* clist )
{
    /* Just a flag to remember the status. 0 = currently visible */
    static short int flag = 0;

    if (flag == 0)
    {
        /* Hide the titles and set the flag to 1 */
	clist->column_titles_hide_ ();
	flag++;
    }
    else
    {
        /* Show the titles and reset flag to 0 */
	clist->column_titles_show_ ();
	flag--;
    }

    return;
}

/* If we come here, then the user has selected a row in the list. */
void selection_made( GtkCList       *clist,
                     gint            row,
                     gint            column,
		     GdkEventButton *event,
                     gpointer        data )
{
    gchar *text;

    /* Get the text that is stored in the selected row and column
     * which was clicked in. We will receive it as a pointer in the
     * argument text.
     */
    clist->get_text_(row, column, &text);

    /* Just prints some information about the selected row */
    g_print("You selected row %d. More specifically you clicked in "
            "column %d, and the text in this cell is %s\n\n",
	    row, column, text);

    return;
}

int main( int    argc,
          gchar *argv[] )
{                                  
    GtkWindow *window;
    GtkBox *vbox, *hbox;
    GtkCList *clist;
    GtkButton *button_add, *button_clear, *button_hide_show;    
    gchar *titles[2] = { "Ingredients", "Amount" };

    gtk_init(&argc, &argv);
    
    window=GtkWindow::_new(GTK_WINDOW_TOPLEVEL);
    window->set_usize_ (300, 150);

    window->set_title_ ("GtkCList Example");
    window->connect_ ("destroy", GTK_SIG gtk_main_quit, NULL);
    
    vbox = GtkVBox::_new(FALSE, 5);
    vbox->set_border_width_ (5);
    window->add_ (vbox)->show_ ();
    
    /* Create the GtkCList. For this example we use 2 columns */
    clist = GtkCList::_new_with_titles( 2, titles);

    /* When a selection is made, we want to know about it. The callback
     * used is selection_made, and its code can be found further down */
    clist->connect_("select_row", GTK_SIG selection_made, NULL);

    /* It isn't necessary to shadow the border, but it looks nice :) */
    clist->set_shadow_type_ (GTK_SHADOW_OUT);

    /* What however is important, is that we set the column widths as
     * they will never be right otherwise. Note that the columns are
     * numbered from 0 and up (to 1 in this case).
     */
    clist->set_column_width_ (0, 150);

    /* Add the GtkCList widget to the vertical box and show it. */
    vbox->pack_start_ (clist, TRUE, TRUE, 0)->show_  ();

    /* Create the buttons and add them to the window. See the button
     * tutorial for more examples and comments on this.
     */
    hbox = GtkHBox::_new(FALSE, 0);
    vbox->pack_start_ (hbox, FALSE, TRUE, 0)->show_ ();

    button_add = GtkButton::_new_with_label("Add List");
    button_clear = GtkButton::_new_with_label("Clear List");
    button_hide_show = GtkButton::_new_with_label("Hide/Show titles");

    hbox->pack_start_ (button_add, TRUE, TRUE, 0)->show_ ();
    hbox->pack_start_ (button_clear, TRUE, TRUE, 0)->show_ ();
    hbox->pack_start_ (button_hide_show, TRUE, TRUE, 0)->show_ ();

    /* Connect our callbacks to the three buttons */
    button_add->connect_object_ ("clicked",
			      GTK_SIG button_add_clicked, clist);
    button_clear->connect_object_ ("clicked",
			      GTK_SIG button_clear_clicked, clist);
    button_hide_show->connect_object_ ("clicked",
			      GTK_SIG button_hide_show_clicked, clist);

    /* The interface is completely set up so we show the window and
     * enter the gtk_main loop.
     */
    window->show_ ();
    gtk_main();
    
    return(0);
}
/* example-end */
