
/* example-start packbox packbox.c */
#include <stdlib.h>
#include <stdio.h>
#include <gtk/cc/gtk.h>

void delete_event( GtkWidget *widget,  GdkEvent  *event,  gpointer   data )
{
  gtk_main_quit ();
}

/* Make a new hbox filled with button-labels. Arguments for the 
* variables we're interested are passed in to this function. 
* We do not show the box, but do show everything inside. */
GtkBox *make_box( gint homogeneous,  gint spacing, gint expand, gint fill,  gint padding ) 
{
  GtkBox *box;
  GtkButton *button;
  char padstr[80];
  
  /* Create a new hbox with the appropriate homogeneous
   * and spacing settings */
  box = GtkHBox::_new (homogeneous, spacing);
  
  /* Create a series of buttons with the appropriate settings */
  button = GtkButton::_new_with_label ("gtk_box_pack");
  box->pack_start_(button, expand, fill, padding);
  button->show_();
  
  button = GtkButton::_new_with_label ("(box,");
  box->pack_start_(button, expand, fill, padding);
  button->show_();
  
  button = GtkButton::_new_with_label ("button,");
  box->pack_start_(button, expand, fill, padding);
  button->show_();
  
  /* Create a button with the label depending on the value of
   * expand. */
  if (expand == TRUE)
          button = GtkButton::_new_with_label ("TRUE,");
  else
          button = GtkButton::_new_with_label ("FALSE,");
  
  box->pack_start_(button, expand, fill, padding);
  button->show_();
  
  /* This is the same as the button creation for "expand"
   * above, but uses the shorthand form. */
  button = GtkButton::_new_with_label (fill ? "TRUE," : "FALSE,");
  box->pack_start_(button, expand, fill, padding);
  button->show_();
  
  sprintf (padstr, "%d);", padding);
  
  button = GtkButton::_new_with_label (padstr);
  box->pack_start_(button, expand, fill, padding);
  button->show_();
  
  return box;
}

int main( int argc, char *argv[]) 
{
  GtkWindow *window;
  GtkButton *button;
  GtkBox *box1;
  GtkBox *box2;
  GtkSeparator *separator;
  GtkLabel *label;
  GtkBox *quitbox;
  int which;
  
  /* Our init, don't forget this! :) */
  gtk_init (&argc, &argv);
  
  if (argc != 2) {
      fprintf (stderr, "usage: packbox num, where num is 1, 2, or 3.\n");
      /* This just does cleanup in GTK and exits with an exit status of 1. */
      gtk_exit (1);
  }
  
  which = atoi (argv[1]);

  /* Create our window */
  window = GtkWindow::_new (GTK_WINDOW_TOPLEVEL);

  /* You should always remember to connect the destroy signal to the
   * main window. This is very important for proper intuitive
   * behavior */
  window->connect_("delete_event", GTK_SIG delete_event, NULL);
  window->set_border_width_(10);
  
  /* We create a vertical box (vbox) to pack the horizontal boxes into.
   * This allows us to stack the horizontal boxes filled with buttons one
   * on top of the other in this vbox. */
  box1 = GtkVBox::_new (FALSE, 0);
  
  /* which example to show. These correspond to the pictures above. */
  switch (which) {
  case 1:
      /* create a new label. */
      label = GtkLabel::_new ("gtk_hbox_new (FALSE, 0);");
      
      /* Align the label to the left side.  We'll discuss this function and 
       * others in the section on Widget Attributes. */
      label->set_alignment_(0, 0);

      /* Pack the label into the vertical box (vbox box1).  Remember that 
       * widgets added to a vbox will be packed one on top of the other in
       * order. */
      box1->pack_start_(label, FALSE, FALSE, 0);
      
      /* Show the label */
      label->show_();
      
      /* Call our make box function - homogeneous = FALSE, spacing = 0,
       * expand = FALSE, fill = FALSE, padding = 0 */
      box2 = make_box (FALSE, 0, FALSE, FALSE, 0);
      box1->pack_start_(box2, FALSE, FALSE, 0);
      box2->show_();

      /* Call our make box function - homogeneous = FALSE, spacing = 0,
       * expand = FALSE, fill = FALSE, padding = 0 */
      box2 = make_box (FALSE, 0, TRUE, FALSE, 0);
      box1->pack_start_(box2, FALSE, FALSE, 0);
      box2->show_();
      
      /* Args are: homogeneous, spacing, expand, fill, padding */
      box2 = make_box (FALSE, 0, TRUE, TRUE, 0);
      box1->pack_start_(box2, FALSE, FALSE, 0);
      box2->show_();
      
      /* Creates a separator, we'll learn more about these later, 
       * but they are quite simple. */
      separator = GtkHSeparator::_new ();
      
      /* Cack the separator into the vbox. Remember each of these
       * widgets are being packed into a vbox, so they'll be stacked
       * vertically. */
      box1->pack_start_(separator, FALSE, TRUE, 5);
      separator->show_();
      
      /* Create another new label, and show it. */
      label = GtkLabel::_new ("gtk_hbox_new (TRUE, 0);");
      gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
      gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 0);
      gtk_widget_show (label);
      
      /* Args are: homogeneous, spacing, expand, fill, padding */
      box2 = make_box (TRUE, 0, TRUE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, FALSE, 0);
      gtk_widget_show (box2);
      
      /* Args are: homogeneous, spacing, expand, fill, padding */
      box2 = make_box (TRUE, 0, TRUE, TRUE, 0);
      gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, FALSE, 0);
      gtk_widget_show (box2);
      
      /* Another new separator. */
      separator = GtkHSeparator::_new ();
      /* The last 3 arguments to gtk_box_pack_start are:
       * expand, fill, padding. */
      gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 5);
      gtk_widget_show (separator);
      
      break;

  case 2:

      /* Create a new label, remember box1 is a vbox as created 
       * near the beginning of main() */
      label = GtkLabel::_new ("gtk_hbox_new (FALSE, 10);");
      gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
      gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 0);
      gtk_widget_show (label);
      
      /* Args are: homogeneous, spacing, expand, fill, padding */
      box2 = make_box (FALSE, 10, TRUE, FALSE, 0);
      gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, FALSE, 0);
      gtk_widget_show (box2);
      
      /* Args are: homogeneous, spacing, expand, fill, padding */
      box2 = make_box (FALSE, 10, TRUE, TRUE, 0);
      gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, FALSE, 0);
      gtk_widget_show (box2);
      
      separator = GtkHSeparator::_new ();
      /* The last 3 arguments to gtk_box_pack_start are:
       * expand, fill, padding. */
      gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 5);
      gtk_widget_show (separator);
      
      label = GtkLabel::_new ("gtk_hbox_new (FALSE, 0);");
      gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
      gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 0);
      gtk_widget_show (label);
      
      /* Args are: homogeneous, spacing, expand, fill, padding */
      box2 = make_box (FALSE, 0, TRUE, FALSE, 10);
      gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, FALSE, 0);
      gtk_widget_show (box2);
      
      /* Args are: homogeneous, spacing, expand, fill, padding */
      box2 = make_box (FALSE, 0, TRUE, TRUE, 10);
      gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, FALSE, 0);
      gtk_widget_show (box2);
      
      separator = GtkHSeparator::_new ();
      /* The last 3 arguments to gtk_box_pack_start are: expand, fill, padding. */
      gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 5);
      gtk_widget_show (separator);
      break;
  
  case 3:

      /* This demonstrates the ability to use gtk_box_pack_end() to
       * right justify widgets. First, we create a new box as before. */
      box2 = make_box (FALSE, 0, FALSE, FALSE, 0);

      /* Create the label that will be put at the end. */
      label = GtkLabel::_new ("end");
      /* Pack it using gtk_box_pack_end(), so it is put on the right
       * side of the hbox created in the make_box() call. */
      gtk_box_pack_end (GTK_BOX (box2), label, FALSE, FALSE, 0);
      /* Show the label. */
      gtk_widget_show (label);
      
      /* Pack box2 into box1 (the vbox remember ? :) */
      gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, FALSE, 0);
      gtk_widget_show (box2);
      
      /* A separator for the bottom. */
      separator = GtkHSeparator::_new ();
      /* This explicitly sets the separator to 400 pixels wide by 5 pixels
       * high. This is so the hbox we created will also be 400 pixels wide,
       * and the "end" label will be separated from the other labels in the
       * hbox. Otherwise, all the widgets in the hbox would be packed as
       * close together as possible. */
      gtk_widget_set_usize (separator, 400, 5);
      /* pack the separator into the vbox (box1) created near the start 
       * of main() */
      gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 5);
      gtk_widget_show (separator);    
  }
  
  /* Create another new hbox.. remember we can use as many as we need! */
  quitbox = GtkHBox::_new (FALSE, 0);
  
  /* Our quit button. */
  button = GtkButton::_new_with_label ("Quit");
  
  /* Setup the signal to destroy the window. Remember that this will send
   * the "destroy" signal to the window which will be caught by our signal
   * handler as defined above. */
  button->connect_object_("clicked", GTK_SIG gtk_main_quit, window);
  /* Pack the button into the quitbox.
   * The last 3 arguments to gtk_box_pack_start are:
   * expand, fill, padding. */
  quitbox->pack_start_(button, TRUE, FALSE, 0);
  /* pack the quitbox into the vbox (box1) */
  box1->pack_start_(quitbox, FALSE, FALSE, 0);
  
  /* Pack the vbox (box1) which now contains all our widgets, into the
   * main window. */
  window->add_(box1);
  
  /* And show everything left */
  button->show_();
  quitbox->show_();
  
  box1->show_();
  /* Showing the window last so everything pops up at once. */
  window->show_();
  
  /* And of course, our main function. */
  gtk_main ();

  /* Control returns here when gtk_main_quit() is called, but not when 
   * gtk_exit is used. */
  
  return(0);
}
/* example-end */

