#include <math.h>
#include <stdio.h>
#include <gtk/cc/gtk.h>
#include <gtkextra/cc/gtkplotcanvas.h>
#include <gtkextra/cc/gtkplotps.h>

GdkPixmap *pixmap=NULL;
GtkPlot **plots;
GtkToggleButton **buttons;
GtkPlotData *dataset[5];
gint nlayers = 0;

void
quit ()
{
  gtk_main_quit();
}

gdouble function(GtkPlot *plot, GtkPlotData *data, gdouble x, gboolean *err)
{
 *err = FALSE;

 return (-.5+.3*sin(3.*x)*sin(50.*x));
}

gdouble gaussian(GtkPlot *plot, GtkPlotData *data, gdouble x, gboolean *err)
{
 gdouble y;
 *err = FALSE;
 y = .65*exp(-.5*pow(x-.5,2)/.02);
 return y;
}


gint
select_item(GtkPlotCanvas *canvas, GdkEvent *event, GtkPlotCanvasItem *item, 
            gpointer data)
{
  GtkPlot **widget_list = NULL;
  GtkPlot *active_widget = NULL;
  gint n = 0;

  switch(item->type){
    case GTK_PLOT_CANVAS_TEXT:
        printf("Item selected: TEXT\n");
        break;
    case GTK_PLOT_CANVAS_TITLE:
        printf("Item selected: TITLE\n");
        break;
    case GTK_PLOT_CANVAS_LEGENDS:
        printf("Item selected: LEGENDS\n");
        break;
    case GTK_PLOT_CANVAS_PLOT:
        printf("Item selected: PLOT\n");
        break;
    case GTK_PLOT_CANVAS_AXIS:
        printf("Item selected: AXIS\n");
        break;
    case GTK_PLOT_CANVAS_DATA:
        printf("Item selected: DATA\n");
        break;
    case GTK_PLOT_CANVAS_NONE:
        printf("Item selected: NONE\n");
        break;
  }

  widget_list = plots;
  active_widget = canvas->get_active_plot_ ();

  while(n < nlayers)
    {
      if(widget_list[n] == active_widget){
            canvas->set_active_plot_ (plots[n]);  
            buttons[n]->button_down = TRUE;
            buttons[n]->active = TRUE;
            buttons[n]->set_state_ (GTK_STATE_ACTIVE);
      }else{
            buttons[n]->button_down = FALSE;
            buttons[n]->active = FALSE;
            buttons[n]->set_state_ (GTK_STATE_NORMAL);
      }
      buttons[n]->queue_draw_ ();
      
      n++;
    }


  return TRUE;
}

gint
activate_plot(GtkButton *widget, GtkPlotCanvas* canvas)
{
  GtkButton **widget_list = NULL;
  GtkButton *active_widget = NULL;
  gint n = 0;

#define BUG1 (GtkButton**)
  widget_list = BUG1 buttons;
  active_widget = widget;

  while(n < nlayers)
    {
      if(widget_list[n] == active_widget){
            canvas->set_active_plot_ (plots[n]);  
            buttons[n]->button_down = TRUE;
            buttons[n]->active = TRUE;
            buttons[n]->set_state_ (GTK_STATE_ACTIVE);
      }else{
            buttons[n]->button_down = FALSE;
            buttons[n]->active = FALSE;
            buttons[n]->set_state_ (GTK_STATE_NORMAL);
      }
      buttons[n]->queue_draw_ ();
      
      n++;
    }

  return TRUE;
}


GtkPlot *
new_layer(GtkPlotCanvas *canvas)
{
 gchar label[10];

 nlayers++;

 buttons = (GtkToggleButton **)g_realloc(buttons, nlayers * sizeof(GtkToggleButton *));
 plots = (GtkPlot **)g_realloc(plots, nlayers * sizeof(GtkPlot *));

 sprintf(label, "%d", nlayers);
 
 buttons[nlayers-1] = GtkToggleButton::_new_with_label(label);
/* buttons[nlayers-1]->set_relief_ (GTK_RELIEF_NONE);
*/
 buttons[nlayers-1]->set_usize_ (20, 20);
 canvas->put_ (buttons[nlayers-1], (nlayers-1)*20, 0)->show_ ();

 buttons[nlayers-1]->connect_ ("toggled", GTK_SIG activate_plot, canvas);

 plots[nlayers-1] = GtkPlot::_new_with_size(NULL, .5, .25);
 plots[nlayers-1]->show_ ();

 activate_plot(buttons[nlayers-1],canvas);

 return plots[nlayers-1];
}


void
build_example1(GtkPlot *active_plot)
{
 GdkColor color;
 static gdouble px1[]={0., 0.2, 0.4, 0.6, 0.8, 1.0};
 static gdouble py1[]={.2, .4, .5, .35, .30, .40};
 static gdouble dx1[]={.2, .2, .2, .2, .2, .2};
 static gdouble dy1[]={.1, .1, .1, .1, .1, .1};

 static gdouble px2[]={0., -0.2, -0.4, -0.6, -0.8, -1.0};
 static gdouble py2[]={.2, .4, .5, .35, .30, .40};
 static gdouble dx2[]={.2, .2, .2, .2, .2, .2};
 static gdouble dy2[]={.1, .1, .1, .1, .1, .1};

 dataset[0] = GtkPlotDataset::_new();
 active_plot->add_dataset_ (dataset[0]);
 dataset[0]->set_points_ (px1, py1, dx1, dy1, 6);
 gdk_color_parse("red", &color);
 gdk_color_alloc(gdk_colormap_get_system(), &color); 

 dataset[0]->set_symbol_ (GTK_PLOT_SYMBOL_DIAMOND, GTK_PLOT_SYMBOL_OPAQUE, 10, 2, &color);
 dataset[0]->set_line_attributes_ (GTK_PLOT_LINE_SOLID, 1, &color);

 dataset[0]->set_connector_ (GTK_PLOT_CONNECT_SPLINE);

 dataset[0]->show_yerrbars_ ();
 dataset[0]->set_legend_ ("Spline + EY");


 dataset[3] = GtkPlotDataset::_new();
 active_plot->add_dataset_ (dataset[3]);
 dataset[3]->set_points_ (px2, py2, dx2, dy2, 6);
 dataset[3]->set_symbol_ (GTK_PLOT_SYMBOL_SQUARE, GTK_PLOT_SYMBOL_OPAQUE, 8, 2, 
     								&active_plot->style->black);
 dataset[3]->set_line_attributes_(GTK_PLOT_LINE_SOLID, 4, &color);
 dataset[3]->set_connector_ (GTK_PLOT_CONNECT_STRAIGHT);
 dataset[3]->set_x_attributes_ (GTK_PLOT_LINE_SOLID, 0, &active_plot->style->black);
 dataset[3]->set_y_attributes_ (GTK_PLOT_LINE_SOLID, 0, &active_plot->style->black);
 dataset[3]->set_legend_ ("Line + Symbol");
 
 gdk_color_parse("blue", &color);
 gdk_color_alloc(gdk_colormap_get_system(), &color); 

 dataset[1] = active_plot->add_function_ ((GtkPlotFunc)function);
 dataset[1]->set_line_attributes_ (GTK_PLOT_LINE_SOLID,  0, &color);
 dataset[1]->set_legend_("Function Plot");
}

void
build_example2(GtkPlot *active_plot)
{
 GdkColor color;
 static double px2[] = {.1, .2, .3, .4, .5, .6, .7, .8};
 static double py2[] = {.012, .067, .24, .5, .65, .5, .24, .067};
 static double dx2[] = {.1, .1, .1, .1, .1, .1, .1, .1};


 dataset[4] = active_plot->add_function_ ((GtkPlotFunc)gaussian);
 gdk_color_parse("dark green", &color);
 gdk_color_alloc(gdk_colormap_get_system(), &color); 
 dataset[4]->set_line_attributes_ (GTK_PLOT_LINE_DASHED, 2, &color);
 dataset[4]->set_legend_ ("Gaussian");

 gdk_color_parse("blue", &color);
 gdk_color_alloc(gdk_colormap_get_system(), &color); 
/* active_plot->xscale = GTK_PLOT_SCALE_LOG10;
*/
 dataset[2] = GtkPlotDataset::_new();
 active_plot->add_dataset_ (dataset[2]);
 dataset[2]->set_points_ (px2, py2, dx2, NULL, 8);
 dataset[2]->set_symbol_ (GTK_PLOT_SYMBOL_IMPULSE, GTK_PLOT_SYMBOL_FILLED,  10, 5, &color);
 dataset[2]->set_line_attributes_ (GTK_PLOT_LINE_SOLID, 5, &color);

 dataset[2]->set_connector_ (GTK_PLOT_CONNECT_NONE);
 dataset[2]->set_legend_ ("Impulses");
}

int main(int argc, char *argv[]){

 GtkWindow *		window1;
 GtkVBox *		vbox1;
 GtkScrolledWindow *	scrollw1;
 GtkLayout *		layout;
 GtkPlot *		active_plot;
 GtkPlotCanvas *	canvas;
 GdkColor color;
 gint page_width, page_height;
 gfloat scale = 1.;
 
 page_width = GTK_PLOT_LETTER_W * scale;
 page_height = GTK_PLOT_LETTER_H * scale;
 
 gtk_init(&argc,&argv);

 window1 = GtkWindow::_new (GTK_WINDOW_TOPLEVEL);
 window1->set_title_ ("GtkPlot Demo");
 window1->set_usize_ (550, 650);
 window1->set_border_width_ (0);

 window1->connect_ ("destroy", GTK_SIG quit, NULL);

 vbox1 = GtkVBox::_new(FALSE,0);
 window1->add_ (vbox1)->show_ ();

 scrollw1 = GtkScrolledWindow::_new(NULL, NULL);
 scrollw1->set_border_width_ (0);
 scrollw1->set_policy_ (GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);
 vbox1->pack_start_ (scrollw1, TRUE, TRUE,0)->show_ ();

 canvas = GtkPlotCanvas::_new(page_width, page_height);
 GTK_PLOT_CANVAS_SET_FLAGS(canvas, GTK_PLOT_CANVAS_DND_FLAGS);
 layout = canvas;
 scrollw1->add_(layout);
 layout->set_size_(page_width, page_height);
 layout->hadjustment->step_increment = 5;
 layout->vadjustment->step_increment = 5;

/*
 gdk_color_parse("light blue", &color);
 gdk_color_alloc(gtk_widget_get_colormap(layout), &color);
 layout->set_background_ (&color);
*/

 layout->show_ ();
# define BUG3 (GtkOrientation)

 active_plot = new_layer(canvas);
 active_plot->set_range_ (-1. ,1., -1., 1.4);
 active_plot->legends_move_ (.500, .05);
 active_plot->set_legends_border_ (GTK_PLOT_BORDER_NONE, 0);
 active_plot->axis_hide_title_ (GTK_PLOT_AXIS_TOP);
 active_plot->axis_show_ticks_ (GTK_PLOT_AXIS_TOP, 15, 15);
 active_plot->axis_set_ticks_ (BUG3 0, 1., .5);
 active_plot->axis_set_ticks_ (BUG3 1, 1., .5);
 active_plot->axis_set_visible_ (GTK_PLOT_AXIS_TOP, TRUE);
 active_plot->axis_set_visible_ (GTK_PLOT_AXIS_RIGHT, TRUE);
 active_plot->x0_set_visible_ (TRUE);
 active_plot->y0_set_visible_ (TRUE);
 canvas->add_plot_ (active_plot, .15, .06)->show_ ();

 build_example1 (active_plot);

 active_plot = new_layer (canvas);
 gdk_color_parse ("light yellow", &color);
 gdk_color_alloc (gtk_widget_get_colormap(active_plot), &color);
 active_plot->set_background_ (&color);

 gdk_color_parse ("light blue", &color);
 gdk_color_alloc (gtk_widget_get_colormap(layout), &color);
 active_plot->legends_set_attributes_ (NULL, 0, NULL, &color);
 active_plot->set_range_ (0. ,1., 0., .85);
 active_plot->axis_set_visible_ (GTK_PLOT_AXIS_TOP, TRUE);
 active_plot->axis_set_visible_ (GTK_PLOT_AXIS_RIGHT, TRUE);
 active_plot->grids_set_visible_ (TRUE, TRUE, TRUE, TRUE);
 canvas->add_plot_ (active_plot, .15, .4);
 active_plot->axis_hide_title_ (GTK_PLOT_AXIS_TOP);
 active_plot->axis_hide_title_ (GTK_PLOT_AXIS_RIGHT);
 active_plot->set_legends_border_ (GTK_PLOT_BORDER_SHADOW, 3);
 active_plot->legends_move_ (.58, .05);
 active_plot->show_ ();
 build_example2(active_plot);

 canvas->connect_ ("select_item", GTK_SIG select_item, NULL);

 window1->show_ ();

 /* gtk_plot_layout_put_text */
 canvas->put_text_ (.40, .020, 0,  "Times-BoldItalic", 16, NULL, NULL,
                          GTK_JUSTIFY_CENTER,
                          "DnD titles, legends and plots");
 canvas->put_text_ (.40, .72, 0,   "Times-Roman", 16, NULL, NULL,
                          GTK_JUSTIFY_CENTER,
                          "You can use \\ssubscripts\\b\\b\\b\\b\\b\\b\\b\\b\\b\\b\\N\\Ssuperscripts");
 canvas->put_text_ (.40, .745, 0,  "Times-Roman", 16, NULL, NULL,
                          GTK_JUSTIFY_CENTER,
                          "Format text mixing \\Bbold \\N\\i, italics, \\ggreek \\4\\N and \\+different fonts");
 gtk_plot_layout_export_ps (canvas, "plotdemo.ps", 0, 0, GTK_PLOT_LETTER);

 gtk_main();

 return(0);
}


