#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <gtk/cc/gtk.h>
#include <gtkextra/cc/gtkplot_.h>
#include <gtkextra/cc/gtkplotlayout.h>
#include <gtkextra/cc/gtkplotcanvas.h>

GdkPixmap *pixmap=NULL;
GtkPlot **plots;
GtkPlotDataset *dataset;
gint nlayers = 0;
gint32 timer;
GtkPlotLayout *layout;
GtkPlotCanvas *canvas;
gdouble *px;
gdouble *py;

void
quit ()
{
  gtk_timeout_remove(timer);
  gtk_main_quit();
}


GtkPlot *
new_layer(GtkWidget *layout)
{
 gchar label[10];

 nlayers++;

 plots = (GtkPlot **)g_realloc(plots, nlayers * sizeof(GtkPlot *));

 sprintf(label, "%d", nlayers);
 
 plots[nlayers-1] = GtkPlot::_new_with_size(NULL, .65, .45);

 return plots[nlayers-1];
}

gint
update(GtkPlot* plot)
{
 gdouble x, y;
 gdouble xmin, xmax;

 px = (gdouble *)g_realloc(px, (dataset->num_points+1)*sizeof(gdouble));
 py = (gdouble *)g_realloc(py, (dataset->num_points+1)*sizeof(gdouble));

 y = rand()%10/10.;

 if(dataset->num_points == 0)
   x = 1;
 else
   x = px[dataset->num_points-1] + 1;

 px[dataset->num_points] = x;
 py[dataset->num_points] = y;

 dataset->set_numpoints_ (dataset->num_points+1); 
 dataset->set_x_ (px); 
 dataset->set_y_ (py); 

 plot->get_xrange_ (&xmin , &xmax);

 if(x > xmax)
   plot->set_xrange_ (xmin + 5. , xmax + 5.);

 /* gtk_plot_layout_refresh */
 canvas->refresh_ ();

 return TRUE;
}

int main(int argc, char *argv[])
{
 GtkWindow *		window1;
 GtkVBox *		vbox1;
 GtkScrolledWindow *	scrollw1;
 GtkPlot *		active_plot;
 GdkColor color;
 gint page_width, page_height;
 gfloat scale = 1.;
 
 page_width = GTK_PLOT_LETTER_W * scale;
 page_height = GTK_PLOT_LETTER_H * scale;
 
 gtk_init(&argc,&argv);

 window1 = GtkWindow::_new (GTK_WINDOW_TOPLEVEL);
 window1->set_title_ ("GtkPlot Real Time Demo");
 window1->set_usize_ (550, 600);
 window1->set_border_width_ (0);

 window1->connect_ ("destroy", GTK_SIG quit, NULL);

 vbox1 = GtkVBox::_new (FALSE, 0);
 window1->add_ (vbox1)->show_ ();

 scrollw1 = GtkScrolledWindow::_new(NULL, NULL);
 scrollw1->set_border_width_ (0);
 scrollw1->set_policy_ (GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);
 vbox1->pack_start_ (scrollw1, TRUE, TRUE,0)->show_ ();

 canvas = GtkPlotCanvas::_new (page_width, page_height);
 GTK_PLOT_CANVAS_UNSET_FLAGS (canvas, GTK_PLOT_CANVAS_DND_FLAGS);
 layout = canvas;
 scrollw1->add_ (layout);
 layout->set_size_ (page_width, page_height);
 layout->hadjustment->step_increment = 5;
 layout->vadjustment->step_increment = 5;

 gdk_color_parse("light blue", &color);
 gdk_color_alloc(layout->get_colormap_ (), &color);
 layout->set_background_ (&color);

 layout->show_ ();

 active_plot = new_layer(layout);
 gdk_color_parse("light yellow", &color);
 gdk_color_alloc(active_plot->get_colormap_ (), &color);
 active_plot->set_background_ (&color);

#define BUG1 (GtkOrientation)
 gdk_color_parse("white", &color);
 gdk_color_alloc(layout->get_colormap_ (), &color);
 active_plot->legends_set_attributes_ (NULL, 0, NULL, &color);
 active_plot->set_range_ (0. ,20., 0., 1.);
 active_plot->axis_set_ticks_ (BUG1 0, 2, 1);
 active_plot->axis_set_ticks_ (BUG1 1, .1, 1);
 active_plot->axis_labels_set_numbers_ (GTK_PLOT_AXIS_TOP, 0, 0);
 active_plot->axis_labels_set_numbers_ (GTK_PLOT_AXIS_BOTTOM, 0, 0);
 active_plot->axis_set_visible_ (GTK_PLOT_AXIS_TOP, TRUE);
 active_plot->axis_set_visible_ (GTK_PLOT_AXIS_RIGHT, TRUE);
 active_plot->grids_set_visible_ (TRUE, TRUE, TRUE, TRUE);
 canvas->add_plot_ (active_plot, .15, .15);
 active_plot->axis_hide_title_ (GTK_PLOT_AXIS_TOP);
 active_plot->axis_hide_title_ (GTK_PLOT_AXIS_RIGHT);
 active_plot->axis_set_title_ (GTK_PLOT_AXIS_LEFT, "Intensity");
 active_plot->axis_set_title_ (GTK_PLOT_AXIS_BOTTOM, "Time (s)");
 active_plot->set_legends_border_ (GTK_PLOT_BORDER_SHADOW, 3);
 active_plot->legends_move_ (.60, .10);
 active_plot->show_ ();

 window1->show_ ();

 /* gtk_plot_layout_put_text */
 canvas->put_text_ (.45, .05, 0,  "Times-BoldItalic", 20, NULL, NULL,
                          GTK_JUSTIFY_CENTER,
                          "Real Time Demo");

 dataset = GtkPlotDataset::_new (); /* _new (active_plot); */
 active_plot->add_dataset_ (dataset);

 gdk_color_parse("red", &color);
 gdk_color_alloc(gdk_colormap_get_system(), &color);

 dataset->set_legend_ ("Random pulse");
 dataset->set_symbol_ (GTK_PLOT_SYMBOL_DIAMOND, GTK_PLOT_SYMBOL_OPAQUE,  10, 2, &color);
 dataset->set_line_attributes_ (GTK_PLOT_LINE_SOLID, 1, &color);

 timer = gtk_timeout_add(1000, update, active_plot);

 gtk_main();

 return(0);
}


