#! perl -w
sub token_c_comments
{ # $text, \%token, $i
    $_[0] =~
        s/ (\/\* (?:[^\*]|[\*](?!\/))+ \*\/) 
         / token_c_comment($1,$_[1], $_[2]) /gsex;
}

sub token_c_comment
{ # $text, \%token, $i
    my $key = "\@ccomment".$_[2]."\@"; ++$_[2];
    ${$_[1]}{$key} = $_[0];
    return $key;
}

sub token_cc_comments
{ # $text, \%token, $i
    $_[0] =~
        s/ (\/\/ .*) 
         / token_cc_comment($1,$_[1], $_[2]) /gmex;
}

sub token_cc_comment
{ # $text, \%token, $i
    my $key = "\@lcomment".$_[2]."\@"; ++$_[2];
    ${$_[1]}{$key} = $_[0];
    return $key;
}

sub token_strings
{ # $text, \%token, $i
    $_[0] =~
        s/ (\" (?:[^\\\"]|\\.)* \") 
         / token_string($1,$_[1], $_[2]) /gsex;
}

sub token_string
{ # $text, \%token, $i
    my $key = "\@s".$_[2]."\@"; ++$_[2];
    ${$_[1]}{$key} = $_[0];
    return $key;
}

sub token_chars
{ # $text, \%token, $i
    $_[0] =~
        s/ (\' (?:[^\\\']|\\.)* \') 
         / token_char($1,$_[1], $_[2]) /gsex;
}

sub token_char
{ # $text, \%token, $i
    my $key = "\@c".$_[2]."\@"; ++$_[2];
    ${$_[1]}{$key} = $_[0];
    return $key;
}

sub token_extern_langs
{ # $text, \%token, $i
    $_[0] =~
        s/ (extern \s* \"[^\\\"]+\") 
         / token_extern_lang($1,$_[1], $_[2]) /gsex;
}

sub token_extern_lang
{ # $text, \%token, $i
    my $key = "\@extern".$_[2]."\@"; ++$_[2];
    ${$_[1]}{$key} = $_[0];
    return $key;
}

# about defbodies

sub token_any_macro_defbodies
{ # $text, \%token, $i
    $_[0] =~
        s/ (^[\ \t]*\#[\ \t]*(?:define|if|ifdef|ifndef)[\ \t]*\w+\([\s\w\,]*\)[\ \t]?) ((?:[^\\\n]+|\\\n)*) 
         / $1.&token_any_macro_defbody($2,$_[1], $_[2]) /gmex;
}

sub token_any_macro_defbody
{ # $text, \%token, $i
    my $key = "\@mdef".$_[2]."\@"; ++$_[2];
    ${$_[1]}{$key} = $_[0];
    return $key;
}

sub token_any_nonmacro_defbodies
{ # $text, \%token, $i
    $_[0] =~
        s/ (^[\ \t]*\#[\ \t]*(?:define|if|ifdef|ifndef)[\ \t]*\w+[\ \t]) ((?:[^\\\n]+|\\\n)*) 
         / $1.&token_any_nonmacro_defbody($2,$_[1], $_[2]) /gmex;
}

sub token_any_nonmacro_defbody
{ # $text, \%token, $i
    my $key = "\@xdef".$_[2]."\@"; ++$_[2];
    ${$_[1]}{$key} = $_[0];
    return $key;
}

sub token_any_defbodies
{
    token_any_macro_defbodies($_[0],$_[1],$_[2]);
    token_any_nonmacro_defbodies($_[0],$_[1],$_[2]);
}

sub token_iscomplex  
{ 
    return 0 if $_[0] =~ m/^[\s\w\@]*$/; 
    return 1;
}

sub token_complex_macro_defbodies
{# $text, \%token, $i
   $_[0] =~
       s/ (^[\ \t]*\#[\ \t]*(?:define|if|ifdef|ifndef)[\ \t]*\w+\([\s\w\,]*\)[\ \t]?) ((?:[^\\\n]+|\\\n)*) 
        / $1.&token_complex_macro_defbody($2,$_[1], $_[2]) /gmex;
}

sub token_complex_macro_defbody
{ # $text, \%token, $i
    return $_[0] if not token_iscomplex($_[0]);

    my $key = "\@mdef".$_[2]."\@"; ++$_[2];
    ${$_[1]}{$key} = $_[0];
    return $key;
}

sub token_complex_nonmacro_defbodies
{ # $text, \%token, $i
    $_[0] =~
        s/ (^[\ \t]*\#[\ \t]*(?:define|if|ifdef|ifndef)[\ \t]*\w+[\ \t]) ((?:[^\\\n]+|\\\n)*) 
         / $1.&token_complex_nonmacro_defbody($2,$_[1], $_[2]) /gmex;
}

sub token_complex_nonmacro_defbody
{ # $text, \%token, $i
    return $_[0] if not token_iscomplex($_[0]);

    my $key = "\@xdef".$_[2]."\@"; ++$_[2];
    ${$_[1]}{$key} = $_[0];
    return $key;
}

sub token_complex_defbodies
{
    token_complex_macro_defbodies($_[0],$_[1],$_[2]);
    token_complex_nonmacro_defbodies($_[0],$_[1],$_[2]);
}

sub token_mark_cpplines
{
    return # number of replacements
        $_[0] =~
            s/ (^ [\ \t]*\#)( .*) $
             / $1."\[".$2."\]" /mgex;
}

sub token_unmark_cpplines
{
    return # number of replacements
        $_[0] =~
            s/ (^ [\ \t]*\#) (\[) ([^\]]*) (\])
             / $1.$3 /mgex;
}

# just the reverse of token_c_simplify
sub token_c_unsimplify
{ # $text, \%token
    token_unmark_cpplines($_[0]);
    return token_c_untokenizes(@_);
}

sub token_c_simplify
{ #text, \%token, $i
    token_c_tokenizes(@_);
    token_mark_cpplines($_[0]);
}

# tokenize C sourcecode to be better parsable
sub token_c_tokenizes
{ # $text, \%token, $i
    token_c_comments(@_);
    token_cc_comments(@_);
    token_extern_langs(@_);
    token_strings(@_);
    token_chars(@_);
    token_complex_defbodies(@_);
}


# just the reverse of token_c_tokenize
sub token_c_untokenizes
{ # $text, \%token 
    my $i = 0; my $c = 0;
    # untokenize multiple times since defbodies could contain @tokens@
    while ($i = token_untokenizes ($_[0],$_[1]))
        { $c += $i; }
    return $c; # number of replacements
}

sub token_untokenizes
{ # $text, \%token
    return # number of replacements
        $_[0] =~
            s/ (\@ [a-zA-Z]+ [0-9]+ \@)
             / &token_untokenize1($1,$_[1]) /gmex;
}

sub token_untokenize1
{ # $token, \%token
    if (exists ${$_[1]}{$_[0]})
         { return ${$_[1]}{$_[0]}; }
    else { return $_[1]; }
}

# returns the untokenized version of the $_[0]
sub token_untokenize
{ # args: $text \%token
    $t = $_[0];

    # untokenize multiple times since there could be defbodies with @tokens@

    while ( ($t =~
        s/ (\@ [a-zA-Z]+ [0-9]+ \@)
         / &token_untokenize1($1,$_[1]) /gmex))
    { }

    return $t;
}

sub token_write_hash
{ # filename, \%token
    my $c; my $k;
    my $F = $_[0];
    
    $c = open F, ">$F";
    return $c if not $c;

    for $k (sort keys %{$_[1]})
    {
        print F "\n",$k;
        my $t = ${$_[1]}{$k};      print "[undefined token{\"$k\"}]" if not defined $t;
        $t =~ s/ (^) /$1."     "/gmex;
        print F "\n",$t; 
    }
    close F;
    return 1; # okay
}

###################### the specials #########################
# specials are to be applied *after* mark_cpplines,
# and you should reverse these with special_(c_)untokenize
# before unmark_cpplines and general untokenize.
# the tags of specials are marked with a bigcap char at 
# the start of the tag.

# just the reverse of token_c_tokenize
sub token_c_special_untokenizes
{ # $text, \%token 
    my $i = 0; my $c = 0;
    # untokenize multiple times since defbodies could contain @tokens@
    while ($i = token_special_untokenizes ($_[0],$_[1]))
        { $c += $i; }
    return $c; # number of replacements
}

sub token_special_untokenizes
{ # $text, \%token
    return # number of replacements
        $_[0] =~
            s/ (\@ [A-Z]\w*\d+ \@)
             / &token_untokenize1($1,$_[1]) /gmex;
}

# returns the untokenized version of the $_[0]
sub token_special_untokenize
{ # args: $text \%token
    $t = $_[0];

    # untokenize multiple times since there could be defbodies with @tokens@

    while ( ($t =~
        s/ (\@ [A-Z]\w*\d+ \@)
         / &token_untokenize1($1,$_[1]) /gmex))
    { }

    return $t;
}

# find both of the c_linkage specials
# and cut them out as tokens
sub token_c_special_clinkage
{ # $text, \%token, $i

  $_[0] =~
      s/( \#\[\s* (?:ifdef|if\+defined) \s+__cplusplus\s* (?:\@\wcomment\d+\@\s*)*  \]\s*
          \@extern\d+\@\s* 
          \{\s*            (?:\@\wcomment\d+\@\s*)*
          \#\[\s* endif\s* (?:\@\wcomment\d+\@\s*)* \]
        )/ &token_c_special_clinkage_B($1,$_[1],$_[2]) /gsex;

  $_[0] =~
      s/( \#\[\s* (?:ifdef|if\s+defined) \s+__cplusplus\s* (?:\@\wcomment\d+\@\s*)*  \]\s*
          \}\s*            (?:\@\wcomment\d+\@\s*)*
          \#\[\s* endif\s* (?:\@\wcomment\d+\@\s*)* \]
        )/ &token_c_special_clinkage_E($1,$_[1],$_[2]) /gsex;

}

sub token_c_special_clinkage_B
{ # $text, \%token, $i
    my $key = "\@Bclinkage".$_[2]."\@"; ++$_[2];
    ${$_[1]}{$key} = $_[0];
    return $key;
}

sub token_c_special_clinkage_E
{ # $text, \%token, $i
    my $key = "\@Eclinkage".$_[2]."\@"; ++$_[2];
    ${$_[1]}{$key} = $_[0];
    return $key;
}

# flatten to a single level of {}'s - i.e. they
# are not nested anymore. You should run c_special_clinkage
# beforehand since they introduce another toplevel of {}'s
sub token_c_special_flattens
{ # $text, \%token, $i
    while ((
        $_[0] =~
            s/ (\{ [^{}]*) (\{ [^{}]* \}) 
             / $1.token_c_special_flatten1($2, $_[1], $_[2]) /gsex ))
    { }
}

sub token_c_special_flatten1
{ # $text, \%token, $i
    my $key = "\@Xblock".$_[2]."\@"; ++$_[2];
    ${$_[1]}{$key} = $_[0];
    return $key;
}

# structbodies are extra since they require an extra(!) ';' to
# follow whereas other {}-pairs just stand in the place(!) of
# a single ';'
sub token_c_structbody_flattens
{ # $text, \%token, $i
        $_[0] =~
            s/ (enum|struct|union|class) 
               ( (?:\s*\b\w+)* \s* (?:\@\wcomment\d+\@)? 
                 (?:  \s* (?: \: | \b\_+(?:inherits|implements|extends)\d+ )
                          (?: \([^\(\)]*\) | \s*\b\w+ )
                 )* \s*
               )
               ( \{ [^\{\}]* \} ) 
             / $1.$2.token_c_structbody_flatten1($3, $_[1], $_[2]) /gsex ;
}

sub token_c_structbody_flatten1
{ # $text, \%token, $i
    my $key = "\@Sblock".$_[2]."\@"; ++$_[2];
    ${$_[1]}{$key} = $_[0];
    return $key;
}

# these bodies may stand in place of a ';', so if you scan for ';'
# you should also look for @Pblock@'s ... see structbody and fullflattens.
sub token_c_othersbody_flattens
{ # $text, \%token, $i
       $_[0] =~
            s/ ( \{ [^{}]* \} ) 
             / token_c_othersbody_flatten1($1, $_[1], $_[2]) /gsex;
}

sub token_c_othersbody_flatten1
{ # $text, \%token, $i
    my $key = "\@Pblock".$_[2]."\@"; ++$_[2];
    ${$_[1]}{$key} = $_[0];
    return $key;
}

sub token_c_special_fullsimplify
{ #text, \%token, $i
    token_c_tokenizes(@_);
    token_mark_cpplines($_[0]);
    token_c_special_clinkage(@_);
    token_c_special_fullflattens(@_);
}

# after this routine is run, no {} should be there anymore
# look out for '@Pblock@' that can stand in the place of a ';'
sub token_c_special_fullflattens
{ # $text, \%token, $i
    token_c_special_flattens(@_);
    token_c_structbody_flattens(@_);
    token_c_othersbody_flattens(@_);
}

